unit fppkg_frm;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ComCtrls, ExtCtrls, fppkg_intf, FppkgCommand, Process;

type

  { TFppkgForm }

  TFppkgForm = class(TForm)
    ImageList1: TImageList;
    SettingsMemo: TMemo;
    OutputMemo: TMemo;
    PackageListView: TListView;
    PageControl: TPageControl;
    Splitter: TSplitter;
    PackagesTabSheet: TTabSheet;
    SettingsTabSheet: TTabSheet;
    ToolBar: TToolBar;
    BuildButton: TToolButton;
    CompileButton: TToolButton;
    FixBrokenButton: TToolButton;
    UpdateButton: TToolButton;
    InstallButton: TToolButton;
    CleanButton: TToolButton;
    ArchiveButton: TToolButton;
    DownloadButton: TToolButton;
    procedure ArchiveButtonClick(Sender: TObject);
    procedure BuildButtonClick(Sender: TObject);
    procedure CleanButtonClick(Sender: TObject);
    procedure CompileButtonClick(Sender: TObject);
    procedure DownloadButtonClick(Sender: TObject);
    procedure FixBrokenButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure InstallButtonClick(Sender: TObject);
    procedure PackageListViewClick(Sender: TObject);
    procedure UpdateButtonClick(Sender: TObject);
  private
    { private declarations }
    fppkg: TFppkgPackageList;

    procedure ExecuteCommandOnPackage(Command: string);
    procedure UpdateUI;
  public
    { public declarations }
  end;

var
  FppkgForm: TFppkgForm;

implementation

{$R *.lfm}

function ReplaceLineEndings(const s, NewLineEnds: string): string;
var
  p: Integer;
  StartPos: LongInt;
begin
  Result:=s;
  p:=1;
  while (p<=length(Result)) do begin
    if Result[p] in [#10,#13] then begin
      StartPos:=p;
      if (p<length(Result))
      and (Result[p+1] in [#10,#13]) and (Result[p]<>Result[p+1]) then
        inc(p);
      Result:=copy(Result,1,StartPos-1)+NewLineEnds+copy(Result,p+1,length(Result));
      inc(p,length(NewLineEnds));
    end else begin
      inc(p);
    end;
  end;
end;

function ExecuteFppkgCommand2(const Command: string; Output: TMemo): integer;
const
  READ_BYTES = 2048;
var
  AProcess: TProcess;
  BytesRead: LongInt;
  n: LongInt;
  M: TMemoryStream;

  procedure UpdateMemoFromStream;
  var
    s: string;
  begin
    if BytesRead > 0 then begin
      SetLength(s, BytesRead);
      M.Read(s[1], BytesRead);

      // this woks exactly like Append() only without the newline bug
      Output.SelText := ReplaceLineEndings(s, LineEnding);

      M.SetSize(0);
      BytesRead:=0;
    end;
  end;

begin
  AProcess := TProcess.Create(nil);
  AProcess.CommandLine := FppkgExecutable + ' ' + Command;;
  AProcess.Options := AProcess.Options + [poUsePipes, poStdErrToOutput];
  AProcess.ShowWindow := swoHIDE;
  AProcess.Execute;

  M := TMemoryStream.Create;
  BytesRead := 0;
  Output.Lines.Text := '';

  while AProcess.Running do
  begin
    // make sure we have room
    M.SetSize(BytesRead + READ_BYTES);

    // try reading it
    n := AProcess.Output.Read((M.Memory + BytesRead)^, READ_BYTES);
    if n > 0
    then begin
      Inc(BytesRead, n);
      UpdateMemoFromStream;
      Application.ProcessMessages;
    end
    else
      // no data, wait 100 ms
      Sleep(100);
  end;
  // read last part
  repeat
    // make sure we have room
    M.SetSize(BytesRead + READ_BYTES);
    // try reading it
    n := AProcess.Output.Read((M.Memory + BytesRead)^, READ_BYTES);
    if n > 0
    then begin
      Inc(BytesRead, n);
      UpdateMemoFromStream;
      Application.ProcessMessages;
    end;
  until n <= 0;

  AProcess.Free;
  M.Free;

  Output.Cursor:=crDefault;
end;

{ TFppkgForm }

procedure TFppkgForm.FixBrokenButtonClick(Sender: TObject);
begin
  try
    OutputMemo.Clear;
    OutputMemo.Enabled := False;
    Application.ProcessMessages;
    OutputMemo.Lines.Add('fixbroken');
    ExecuteFppkgCommand2('fixbroken', OutputMemo);
  finally
    OutputMemo.Enabled := True;
  end;
end;

procedure TFppkgForm.CleanButtonClick(Sender: TObject);
begin
  ExecuteCommandOnPackage('clean');
end;

procedure TFppkgForm.CompileButtonClick(Sender: TObject);
begin
  ExecuteCommandOnPackage('compile');
end;

procedure TFppkgForm.DownloadButtonClick(Sender: TObject);
begin
  ExecuteCommandOnPackage('download');
end;

procedure TFppkgForm.ArchiveButtonClick(Sender: TObject);
begin
  ExecuteCommandOnPackage('archive');
end;

procedure TFppkgForm.BuildButtonClick(Sender: TObject);
begin
  ExecuteCommandOnPackage('build');
end;

procedure TFppkgForm.FormCreate(Sender: TObject);
begin
  Caption := 'Free Pascal package manager for Lazarus';
  PageControl.ActivePage := PackagesTabSheet;

  fppkg := TFppkgPackageList.Create;

  UpdateUI;
end;

procedure TFppkgForm.FormDestroy(Sender: TObject);
begin
  fppkg.Free;
end;

procedure TFppkgForm.InstallButtonClick(Sender: TObject);
begin
  ExecuteCommandOnPackage('install');
end;

procedure TFppkgForm.PackageListViewClick(Sender: TObject);
var
  State: boolean;
begin
  if not Assigned(PackageListView.Selected) then
    State := False
  else
    State := PackageListView.Selected.SubItems[1] <> '-';

  BuildButton.Enabled := State;
  CompileButton.Enabled := State;
  InstallButton.Enabled := State;
  CleanButton.Enabled := State;
  ArchiveButton.Enabled := State;
  DownloadButton.Enabled := State;
end;

procedure TFppkgForm.UpdateButtonClick(Sender: TObject);
begin
  ExecuteFppkgCommand('update');
  UpdateUI;
end;

procedure TFppkgForm.UpdateUI;
var
  i: integer;
  li: TListItem;
  s: TMemoryStream;
begin
  fppkg.LoadPackageList;
  PackageListView.Clear;

  for i := 0 to fppkg.PackageCount - 1 do
  begin
    li := PackageListView.Items.Add;
    li.Caption := fppkg.FppkgPackage[i].Name;
    li.SubItems.Add(fppkg.FppkgPackage[i].Installed);
    li.SubItems.Add(fppkg.FppkgPackage[i].Available);
  end;

  s := TMemoryStream.Create;
  try
    SettingsMemo.Clear;
    ExecuteFppkgCommand('listsettings', s);
    s.Position := 0;
    SettingsMemo.Lines.LoadFromStream(s);
  finally
    s.Free;
  end;

  PackageListViewClick(nil);
end;

procedure TFppkgForm.ExecuteCommandOnPackage(Command: string);
var
  pkg: string;
begin
  if not Assigned(PackageListView.Selected) then
    exit;

  try
    pkg := PackageListView.Selected.Caption;

    OutputMemo.Clear;
    OutputMemo.Enabled := False;
    Application.ProcessMessages;
    OutputMemo.Lines.Add(Command + ' ' + pkg);
    ExecuteFppkgCommand2(Command + ' ' + pkg, OutputMemo);
    UpdateUI;
  finally
    OutputMemo.Enabled := True;
  end;
end;

end.

