unit fppkg_intf;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Dialogs, ContNrs, FppkgCommand;

type

  { TFppkgPackage }

  TFppkgPackage = class
    Name: string;
    Installed: string;
    Available: string;
  public
    procedure LoadFromString(APackageEntry: string);
  end;

  { TFppkgPackageList }

  TFppkgPackageList = class
  private
    FPackageCount: integer;
    FPackageList: TFPObjectList;
    msg: TStrings;
    function GetFppkgPackage(index: integer): TFppkgPackage;
    function GetPackageCount: integer;
  public
    constructor Create;
    destructor Destroy; override;

    property FppkgPackage[index: integer]: TFppkgPackage read GetFppkgPackage;
    property PackageCount: integer read GetPackageCount;

    procedure LoadPackageList;
  end;

implementation

procedure TFppkgPackage.LoadFromString(APackageEntry: string);
var
  s: TStrings;
begin
  s := TStringList.Create;
  try
    s.Delimiter := ' ';
    s.DelimitedText := APackageEntry;

    Name := s[0];
    Installed := s[1];
    Available := s[2];
  finally
    s.Free;
  end;
end;

{ TFppkgPackageList }

function TFppkgPackageList.GetFppkgPackage(index: integer): TFppkgPackage;
begin
  Result := TFppkgPackage(FPackageList[index]);
end;

function TFppkgPackageList.GetPackageCount: integer;
begin
  Result := FPackageList.Count;
end;

procedure TFppkgPackageList.LoadPackageList;
var
  Output: TStringStream;
  sOutput: TStrings;
  i: integer;
  FFppkgPackage: TFppkgPackage;
begin
  //clear all old packages
  FPackageList.Clear;

  Output := TStringStream.Create('');
  sOutput := TStringList.Create;
  try
    ExecuteFppkgCommand('list', Output);
    //DumpStream(Output);
    Output.Position := 0;

    sOutput.LoadFromStream(Output);

    //load packages
    //ignore the first row
    for i := 1 to sOutput.Count - 1 do
    begin
      FFppkgPackage := TFppkgPackage.Create;
      FFppkgPackage.LoadFromString(sOutput[i]);
      FPackageList.Add(FFppkgPackage);
    end;
  finally
    Output.Free;
    sOutput.Free;
  end;
end;

constructor TFppkgPackageList.Create;
begin
  inherited Create;
  FPackageList := TFPObjectList.Create(True);
end;

destructor TFppkgPackageList.Destroy;
begin
  FPackageList.Free;
  inherited Destroy;
end;

end.

